@@ -3,7 +3,15 @@ import re
 import sys
 
 from .ansi import AnsiFore, AnsiBack, AnsiStyle, Style
-from .winterm import winterm, WinColor, WinStyle
+from .winterm import WinTerm, WinColor, WinStyle
+from .win32 import windll
+
+if windll is not None:
+    winterm = WinTerm()
+
+
+def is_a_tty(stream):
+    return hasattr(stream, 'isatty') and stream.isatty()
 
 
 class StreamWrapper(object):
@@ -11,10 +19,10 @@ class StreamWrapper(object):
     Wraps a stream (such as stdout), acting as a transparent proxy for all
     attribute access apart from method 'write()', which is delegated to our
     Converter instance.
-    This class contains no other methods or non-double-underscored attributes,
-    to prevent clashes with names of attributes on the wrapped stream object.
     '''
     def __init__(self, wrapped, converter):
+        # double-underscore everything to prevent clashes with names of
+        # attributes on the wrapped stream object.
         self.__wrapped = wrapped
         self.__convertor = converter
 
@@ -33,38 +41,52 @@ class AnsiToWin32(object):
     '''
     ANSI_RE = re.compile('\033\[((?:\d|;)*)([a-zA-Z])')
 
-    def __init__(self, wrapped, autoreset=False):
+    def __init__(self, wrapped, convert=None, strip=None, autoreset=False):
         # The wrapped stream (normally sys.stdout or sys.stderr)
         self.wrapped = wrapped
 
-        # True if we reset colors to defaults after every .write()
+        # should we reset colors to defaults after every .write()
         self.autoreset = autoreset
 
         # create the proxy wrapping our output stream
         self.stream = StreamWrapper(wrapped, self)
 
-        # True if we strip ANSI sequences from our output
-        self.strip = sys.platform.startswith('win')
+        on_windows = sys.platform.startswith('win')
 
-        # True if we should convert ANSI sequences nto win32 calls
-        self.convert = (
-            self.strip and
-            hasattr(self.wrapped, 'isatty') and
-            self.wrapped.isatty())
+        # should we strip ANSI sequences from our output?
+        if strip is None:
+            strip = on_windows
+        self.strip = strip
+
+        # should we should convert ANSI sequences into win32 calls?
+        if convert is None:
+            convert = on_windows and is_a_tty(wrapped)
+        self.convert = convert
 
         # dict of ansi codes to win32 functions and parameters
         self.win32_calls = self.get_win32_calls()
 
-        # true if we are wrapping stderr
+        # are we wrapping stderr?
         self.on_stderr = self.wrapped is sys.stderr
 
 
+    def should_wrap(self):
+        '''
+        True if this class is actually needed. If false, then the output
+        stream will not be affected, nor will win32 calls be issued, so
+        wrapping stdout is not actually required. This will generally be
+        False on non-Windows platforms, unless optional functionality like
+        autoreset has been requested using kwargs to init()
+        '''
+        return self.convert or self.strip or self.autoreset
+
+
     def get_win32_calls(self):
         if self.convert and winterm:
             return {
                 AnsiStyle.RESET_ALL: (winterm.reset_all, ),
                 AnsiStyle.BRIGHT: (winterm.style, WinStyle.BRIGHT),
-                AnsiStyle.DIM: (winterm.style, WinStyle.DIM),
+                AnsiStyle.DIM: (winterm.style, WinStyle.NORMAL),
                 AnsiStyle.NORMAL: (winterm.style, WinStyle.NORMAL),
                 AnsiFore.BLACK: (winterm.fore, WinColor.BLACK),
                 AnsiFore.RED: (winterm.fore, WinColor.RED),
@@ -88,14 +110,22 @@ class AnsiToWin32(object):
 
 
     def write(self, text):
-        if self.strip:
+        if self.strip or self.convert:
             self.write_and_convert(text)
         else:
             self.wrapped.write(text)
+            self.wrapped.flush()
         if self.autoreset:
-            self.write_and_convert(Style.RESET_ALL)
+            self.reset_all()
         
 
+    def reset_all(self):
+        if self.convert:
+            self.call_win32('m', (0,))
+        else:
+            self.wrapped.write(Style.RESET_ALL)
+
+
     def write_and_convert(self, text):
         '''
         Write the given text to our wrapped stream, stripping any ANSI
@@ -140,5 +170,6 @@ class AnsiToWin32(object):
                     func_args = self.win32_calls[param]
                     func = func_args[0]
                     args = func_args[1:]
-                    func(*args, on_stderr=self.on_stderr)
+                    kwargs = dict(on_stderr=self.on_stderr)
+                    func(*args, **kwargs)
 
